from . import VK
from .config import Default
from .yandexmusic import YandexMusic
from .spotify import Spotify
import sys
import argparse
import json
import logging
import sys
import distutils
import os
from shutil import copy2


# logging.basicConfig(filename=Default.LOG_PATH + 'running.log', filemode='w', level=logging.DEBUG,
#                     format='%(asctime)s - %(levelname)s - %(message)s')


# converting strings like 'False True 0 1' to bool
def str2bool(v):
    return bool(distutils.util.strtobool(v))


# проверяем остался ли поврежденный дата файл, если да - удаляем
# копируем и переименовываем файл-экземпляр
# если и он пропал я не виноват
def repair_template(path):
    if os.path.exists(path):
        os.remove(path)
        logging.debug(f"old data file ({path}) deleted")
    if not os.path.exists(str(path) + '.example'):
        logging.error(f"file 'dataTemplate.json.example' was not found")
    else:
        copy2(str(path) + '.example', str(path))
        logging.debug('example template successfully copied')
        return



# clears data file from past records
def clear_template(path=Default.DATATMP):
    logging.debug('cleaning template')
    with open(path) as f:
        try:
            data = json.load(f)
        except json.decoder.JSONDecodeError as jex:
            logging.error(f"Invalid JSON response, dropping to original template \n If the problem persists, check connection to desired service")
            logging.error(jex)
            repair_template(path)
            return
        for i in data:
            for j in data[i]:
                data[i][j].clear()
        with open('dataTemplate.json', 'w', encoding='utf-8') as f:
            json.dump(data, f, indent=2, ensure_ascii=False)
        return data


# Parser for command-line options
def process_args(args, defaults):

    parser = argparse.ArgumentParser()
    parser.prog = 'MMM'

    parser.set_defaults(scope=defaults.SCOPE)  # область необходимых разрешений для работы со spotify API

    parser.set_defaults(data_path=defaults.DATATMP)

    parser.set_defaults(log_path=defaults.LOG_PATH)

    parser.add_argument('-s', '--source', dest='source', required=True, choices=['vk', 'ym', 'sp'], type=str,
                              help='service_name to fetch music from')

    parser.add_argument('--source-user', dest='source_user',
                              type=str, required=True, help='login for source site')

    parser.add_argument('--source-pass', dest='source_pass',
                              type=str, required=True, help='password for source site')

    parser.add_argument('-t', '--target', required=True, dest='target', choices=["ym", "sp"],
                            type=str, help='service_name to export music to')

    parser.add_argument('--target-user', dest='target_user',
                              type=str, required=True, help='login for target site')

    parser.add_argument('--target-pass', dest='target_pass',
                              type=str, required=True, help='password for target site')

    parser.add_argument('--playlists', dest='playlists',
                              action='store_true', default=defaults.PLAYLIST,
                              help=('include playlists as well (default: %s)' % (defaults.PLAYLIST)))

    parser.add_argument('--artists', dest='artists',
                              action='store_true', default=defaults.ARTISTS,
                              help=('include artists as well (default: %s)' % (defaults.ARTISTS)))

    parser.add_argument('--albums', dest='albums',
                              action='store_true', default=defaults.ALBUMS,
                              help=('include albums as well (default: %s)' % (defaults.ALBUMS)))

    parser.add_argument('--alltracks', dest='alltracks',
                              action='store_true',
                              default=defaults.ALLTRACKS,
                              help=('include albums as well (default: %s)' % (defaults.ALLTRACKS)))


    parameters = parser.parse_args(args)
    return parameters


def main(args=None):

    if args is None:
        args = sys.argv[1:]
        print(args)
    parameters = process_args(args, Default)

    if not os.path.exists(parameters.log_path):
        os.mkdir(parameters.log_path)

    logging.basicConfig(filename=parameters.log_path + 'running.log', filemode='w', level=logging.DEBUG,
                        format='%(asctime)s - %(levelname)s - %(message)s')
    data = {
        "YM": {
            "artists": [],
            "albums": [],
            "playlists": {},
            "alltracks": []
        },
        "SP": {
            "artists": [],
            "albums": [],
            "playlists": {},
            "alltracks": []
        },
        "VK": {
            "artists": [],
            "albums": [],
            "playlists": {},
            "alltracks": []
        }
    }

    if parameters.source == "vk":
        vkaudio = VK.get_auth(login=parameters.source_user,
                                password=parameters.source_pass)
        data = case_export(vkaudio,parameters, data)

        if parameters.target == "ym":
            t_obj = YandexMusic(login=parameters.target_user, password=parameters.target_pass, export_data=data, source=parameters.source)
            t_obj.get_auth()
            case_import(t_obj=t_obj, parameters=parameters)

        elif parameters.target == "sp":
            t_obj = Spotify(parameters.target_user, scope=parameters.scope,
                            data=data, source=parameters.source)
            t_obj.get_auth()
            case_import(t_obj=t_obj, parameters=parameters)

    elif parameters.source == 'ym':
        s_obj = YandexMusic(login=parameters.source_user,
                            password=parameters.source_pass, export_data=data)
        s_obj.get_auth()
        data = case_export(s_obj, parameters)
        t_obj = Spotify(parameters.target_user,
                        scope=parameters.scope, data=data, source=parameters.source)
        t_obj.get_auth()
        case_import(t_obj, parameters)

    elif parameters.source == 'sp':
        s_obj = Spotify(parameters.source_user,
                        scope=parameters.scope, data=data)
        s_obj.get_auth()
        t_obj = YandexMusic(login=parameters.target_user,
                            password=parameters.target_pass, export_data=data, source=parameters.source)
        t_obj.get_auth()
        case_import(t_obj,parameters)


# Импорт данных с учетом конфига/командных аргументов
def case_import(t_obj, parameters):
    if parameters.playlists:
        t_obj.import_playlists()
    if parameters.alltracks:
        t_obj.import_alltracks()
    if parameters.artists:
        t_obj.import_artists()
    if parameters.albums:
        t_obj.import_albums()
    return "SELECT IMPORT SUCCEDED"


def case_export(s_obj, parameters, data=None):
    if parameters.playlists:
        if parameters.source == 'vk':
            return VK.export_playlists(s_obj, data)
        else:
            s_obj.export_playlists()
    if parameters.alltracks:
        if parameters.source == 'vk':
            logging.error('vk alltracks export unavailable')    #БАГ API
            sys.exit()
        else:
            s_obj.export_alltracks()
    if parameters.artists:
        s_obj.export_artists()
    if parameters.albums:
        s_obj.export_albums()


    # извлекаем данные для всех, кроме вк. У него лишняя хромосома
    if parameters.source != 'vk':
        data = s_obj.export_data

    return data


if __name__ == '__main__':
    main()



from . import VK
from .spotify import Spotify
from .yandexmusic import YandexMusic
from .config import Default


# Попытка авторизоваться и получить объекты класса + плейлисты соурса
def init_class_objects(source, s_login, s_pass, target, t_login, t_pass):
    data = {
        "YM": {
            "artists": [],
            "albums": [],
            "playlists": {},
            "alltracks": []
        },
        "SP": {
            "artists": [],
            "albums": [],
            "playlists": {},
            "alltracks": []
        },
        "VK": {
            "artists": [],
            "albums": [],
            "playlists": {},
            "alltracks": []
        }
    }
    if source == 'sp':
        sp_obj = Spotify(s_login, scope=Default.SCOPE, data=data)
        s_playlists = sp_obj.get_playlists()
        if target == 'ym':
            ym_obj = YandexMusic(login=t_login, password=t_pass, export_data=data)
        else:
            return 'unexpected target'
        return sp_obj, ym_obj, s_playlists

    elif source == 'ym':
        ym_obj = YandexMusic(login=t_login, password=t_pass, export_data=data)
        s_playlists = ym_obj.get_playlists()
        if target == 'sp':
            sp_obj = Spotify(s_login, scope=Default.SCOPE, data=data)
        else:
            return 'unexpected target'
        return ym_obj, sp_obj, s_playlists

    elif source == 'vk':
        vkaudio = VK.get_auth(s_login,s_pass)
        s_playlists = VK.get_playlists(vkaudio)
        print(s_playlists)
        if target == 'ym':
            t_obj = YandexMusic(login=t_login, password=t_pass, export_data=data)
        elif target == 'sp':
            t_obj = Spotify(s_login, scope=Default.SCOPE, data=data)
        return vkaudio, t_obj, s_playlists



import pytest
from ..MoveMyMusic.yandexmusic import YandexMusic


exoport_data = {
    "YM": {
        "artists": [],
        "albums": [],
        "playlists": {},
        "alltracks": []
    },
    "SP": {
        "artists": [],
        "albums": [],
        "playlists": {},
        "alltracks": []
    },
    "VK": {
        "artists": [],
        "albums": [],
        "playlists": {},
        "alltracks": []
    }
}

import_data = {
    "YM": {
        "artists": ['ЩЕНКИ', 'Nothing but thieves', 'Muse', 'System of a down'],
        "albums": [{"title": "PAYCHECK",
                    "artist": "Пошлая Молли",
                    "tracks_count": 6},
                   {"title": "МОЯ МИЛАЯ ПУСТОТА",
                    "artist": "Pyrokinesis",
                    "tracks_count": 17},
                   {"title": "Корми демонов по расписанию",
                    "artist": "Pyrokinesis",
                    "tracks_count": 8}],
        "playlists": {},
        "alltracks": []
    },
    "SP": {
        "artists": ['ЩЕНКИ', 'Nothing but thieves', 'Muse', 'System of a down'],
        "albums": [{"title": "PAYCHECK",
                    "artist": "Пошлая Молли",
                    "tracks_count": 6},
                   {"title": "МОЯ МИЛАЯ ПУСТОТА",
                    "artist": "Pyrokinesis",
                    "tracks_count": 17},
                   {"title": "Корми демонов по расписанию",
                    "artist": "Pyrokinesis",
                    "tracks_count": 8}],
        "playlists": {},
        "alltracks": []
    },
    "VK": {
        "artists": [],
        "albums": [],
        "playlists": {},
        "alltracks": []
    }
}

login = xxx
password = xxx


def test_export_all():
    ym_obj = YandexMusic(login=login, password=password, export_data=exoport_data)
    ym_obj.get_auth()
    ym_obj.export_alltracks()
    out = ym_obj.export_data
    print(len(out["YM"]["alltracks"]))
    print(out["YM"]["alltracks"][:2])

def test_get_playlists():
    ym_obj = YandexMusic(login=login,
                         password=password, export_data=exoport_data)
    ym_obj.get_auth()
    out = ym_obj.get_playlists()
    print(out)

def test_export_playlists_all():
    ym_obj = YandexMusic(login=login,
                         password=password, export_data=exoport_data)
    ym_obj.get_auth()
    ym_obj.export_playlists()
    out = ym_obj.export_data
    print(len(out["YM"]["playlists"]))


def test_export_playlists():
    ym_obj = YandexMusic(login=login,
                         password=password, export_data=exoport_data)
    ym_obj.get_auth()
    ym_obj.export_playlists(playlist_chosen=None)
    out = ym_obj.export_data
    print(len(out["YM"]["playlists"]))


def test_export_albums():
    ym_obj = YandexMusic(login=login,
                         password=password, export_data=exoport_data)
    ym_obj.get_auth()
    ym_obj.export_albums()
    out = ym_obj.export_data
    print(len(out["YM"]["albums"]))
    print(out["YM"]["albums"][:2])

def test_export_artists():
    ym_obj = YandexMusic(login=login,
                         password=password, export_data=exoport_data)
    ym_obj.get_auth()
    ym_obj.export_artists()
    out = ym_obj.export_data
    print(len(out["YM"]["artists"]))
    print(out["YM"]["artists"][:2])


def test_import_all():
    ym_obj = YandexMusic(login=login,
                         password=password, export_data=import_data, source='sp')
    ym_obj.get_auth()
    ym_obj.import_alltracks()



def test_import_playlists():
    ym_obj = YandexMusic(login=login,
                         password=password, export_data=import_data, source='sp')
    ym_obj.get_auth()
    ym_obj.import_playlists()



def test_import_albums():
    ym_obj = YandexMusic(login=login,
                         password=password, export_data=import_data, source='sp')
    ym_obj.get_auth()
    ym_obj.import_albums()



def test_import_artists():
    ym_obj = YandexMusic(login=login,
                         password=password, export_data=import_data, source='sp')
    ym_obj.get_auth()
    ym_obj.import_artists()



import requests
from yandex_music.client import Client
from yandex_music.utils.difference import Difference
from yandex_music.utils.request import Request
import json
import time
from MoveMyMusic.yandexmusic import YandexMusic


ym = YandexMusic('xxx', 'xxx', export_data=None)
ym.get_auth()


# print(client.token)
# POST
url = 'https://music.yandex.ru/handlers/import.jsx'


playlist_id, playlist_rev, playlist_trackCount = ym.check_playlist('huila')
print(f'ID:{playlist_id}, REV:{playlist_rev}, COUNT:{playlist_trackCount}')
time.sleep(2)
playlist_id, playlist_rev, playlist_trackCount = ym.check_playlist('huila')
print(f'ID:{playlist_id}, REV:{playlist_rev}, COUNT:{playlist_trackCount}')

song_to_import = "хаски ай\nхаски панелька\nхаски бит шатает голову\nпошлая молли супермаркет\nпошлая молли мишка"
string_of_songs = song_to_import.replace(' ', '+')

json_values = {
               'content': string_of_songs
               }

while True:
    r1_import = requests.post(url, json=json_values, allow_redirects=False)
    if r1_import.status_code == 302:
        time.sleep(3)
    if r1_import.status_code == 200:
        id_import = json.loads(r1_import.text)['importCode']
        break
params={
    'code':id_import
}
time.sleep(3)

while True:
    resp = requests.get(url=url, params=params, allow_redirects=False)
    if resp.status_code == 200:
        resp = json.loads(resp.text)
        if resp['status'] == 'in-progress':
            time.sleep(3)
        elif resp['status'] == 'done':
            print('done')
            trackIds = resp['trackIds']
            print(trackIds)
            break
    else:
        print(resp.status_code)
        time.sleep(3)

playlist_id, playlist_rev, playlist_trackCount = ym.check_playlist('huila')
print(f'ID:{playlist_id}, REV:{playlist_rev}, COUNT:{playlist_trackCount}')
tracks_diff = [dict([('id', item.split(':')[0]), ('album_id', item.split(':')[1])]) for item in trackIds]
print(tracks_diff)
diff = Difference().add_insert(playlist_trackCount, tracks_diff)
print(diff)
ym.client.users_playlists_change(kind=playlist_id,diff=diff.to_json(),revision=playlist_rev)
